@@ -292,7 +292,9 @@ static void matchpathcon_thread_destructor(void __attribute__((unused)) *ptr)
 	matchpathcon_fini();
 }
 
-void __attribute__((destructor)) matchpathcon_lib_destructor(void)
+void __attribute__((destructor)) matchpathcon_lib_destructor(void);
+
+void hidden __attribute__((destructor)) matchpathcon_lib_destructor(void)
 {
 	if (destructor_key_initialized)
 		__selinux_key_delete(destructor_key);
@@ -359,11 +361,17 @@ int realpath_not_final(const char *name, char *resolved_path)
 		goto out;
 	}
 
+	/* strip leading // */
+	while (tmp_path[len] && tmp_path[len] == '/' &&
+	       tmp_path[len+1] && tmp_path[len+1] == '/') {
+		tmp_path++;
+		len++;
+	}
 	last_component = strrchr(tmp_path, '/');
 
 	if (last_component == tmp_path) {
 		last_component++;
-		p = strcpy(resolved_path, "/");
+		p = strcpy(resolved_path, "");
 	} else if (last_component) {
 		*last_component = '\0';
 		last_component++;
@@ -398,7 +406,7 @@ out:
 	return rc;
 }
 
-int matchpathcon(const char *path, mode_t mode, security_context_t * con)
+int matchpathcon(const char *path, mode_t mode, char ** con)
 {
 	char stackpath[PATH_MAX + 1];
 	char *p = NULL;
@@ -419,7 +427,7 @@ int matchpathcon(const char *path, mode_t mode, security_context_t * con)
 		selabel_lookup(hnd, con, path, mode);
 }
 
-int matchpathcon_index(const char *name, mode_t mode, security_context_t * con)
+int matchpathcon_index(const char *name, mode_t mode, char ** con)
 {
 	int i = matchpathcon(name, mode, con);
 
@@ -436,8 +444,8 @@ void matchpathcon_checkmatches(char *str __attribute__((unused)))
 
 /* Compare two contexts to see if their differences are "significant",
  * or whether the only difference is in the user. */
-int selinux_file_context_cmp(const security_context_t a,
-			     const security_context_t b)
+int selinux_file_context_cmp(const char * a,
+			     const char * b)
 {
 	char *rest_a, *rest_b;	/* Rest of the context after the user */
 	if (!a && !b)
@@ -459,8 +467,8 @@ int selinux_file_context_cmp(const security_context_t a,
 
 int selinux_file_context_verify(const char *path, mode_t mode)
 {
-	security_context_t con = NULL;
-	security_context_t fcontext = NULL;
+	char * con = NULL;
+	char * fcontext = NULL;
 	int rc = 0;
 
 	rc = lgetfilecon_raw(path, &con);
@@ -498,7 +506,7 @@ int selinux_lsetfilecon_default(const char *path)
 {
 	struct stat st;
 	int rc = -1;
-	security_context_t scontext = NULL;
+	char * scontext = NULL;
 	if (lstat(path, &st) != 0)
 		return rc;
 
