@@ -6,14 +6,16 @@
 #include <strings.h>
 #include <selinux/selinux.h>
 
-void usage(const char *progname) {
-	fprintf(stderr, "usage:  %s [ Enforcing | Permissive | 1 | 0 ]\n", progname);
+static void usage(const char *progname)
+{
+	fprintf(stderr, "usage:  %s [ Enforcing | Permissive | 1 | 0 ]\n",
+		progname);
 	exit(1);
 }
 
-int main(int argc, char **argv) 
+int main(int argc, char **argv)
 {
-	int rc=0;
+	int rc = 0;
 	if (argc != 2) {
 		usage(argv[0]);
 	}
@@ -22,17 +24,14 @@ int main(int argc, char **argv)
 		fprintf(stderr, "%s: SELinux is disabled\n", argv[0]);
 		return 1;
 	}
-	if (strlen(argv[1]) == 1 && (argv[1][0] == '0' || argv[1][0]=='1')) {
+	if (strlen(argv[1]) == 1 && (argv[1][0] == '0' || argv[1][0] == '1')) {
 		rc = security_setenforce(atoi(argv[1]));
 	} else {
-		if (strcasecmp(argv[1], "enforcing")==0) {
+		if (strcasecmp(argv[1], "enforcing") == 0) {
 			rc = security_setenforce(1);
-		}
-		else
-		if (strcasecmp(argv[1], "permissive")==0) {
+		} else if (strcasecmp(argv[1], "permissive") == 0) {
 			rc = security_setenforce(0);
-		}
-		else
+		} else
 			usage(argv[0]);
 	}
 	if (rc < 0) {
